@@ -24,6 +24,7 @@
 #include <KPluginLoader>
 #include <KPluginMetaData>
 
+#include <QCoreApplication>
 #include <QDirIterator>
 #include <qstandardpaths.h>
 
@@ -70,7 +71,7 @@ QList<KProtocolInfoPrivate *> KProtocolInfoFactory::allProtocols()
 KProtocolInfoPrivate *KProtocolInfoFactory::findProtocol(const QString &protocol)
 {
     Q_ASSERT(!protocol.isEmpty());
-    Q_ASSERT(!protocol.contains(':'));
+    Q_ASSERT(!protocol.contains(QLatin1Char(':')));
 
     QMutexLocker locker(&m_mutex);
 
@@ -102,10 +103,12 @@ bool KProtocolInfoFactory::fillCache()
 
     // first: search for meta data protocol info, that might be bundled with applications
     // we search in all library paths inside kf5/kio
-    Q_FOREACH (const KPluginMetaData &md, KPluginLoader::findPlugins("kf5/kio")) {
+    const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kio"));
+    for (const KPluginMetaData &md : plugins) {
         // get slave name & protocols it supports, if any
         const QString slavePath = md.fileName();
         const QJsonObject protocols(md.rawData().value(QStringLiteral("KDE-KIO-Protocols")).toObject());
+        qCDebug(KIO_CORE) << slavePath << "supports protocols" << protocols.keys();
 
         // add all protocols, does nothing if object invalid
         for (auto it = protocols.begin(); it != protocols.end(); ++it) {
@@ -123,7 +126,9 @@ bool KProtocolInfoFactory::fillCache()
     }
 
     // second: fallback to .protocol files
-    Q_FOREACH (const QString &serviceDir, QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QLatin1String("kservices5"), QStandardPaths::LocateDirectory)) {
+    const QStringList serviceDirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kservices5"), QStandardPaths::LocateDirectory)
+        << QCoreApplication::applicationDirPath() + QLatin1String("/kservices5");
+    for (const QString &serviceDir : serviceDirs) {
         QDirIterator it(serviceDir);
         while (it.hasNext()) {
             const QString file = it.next();
