@@ -91,7 +91,11 @@ static bool forkSlaves()
     // In such case we start the slave via QProcess.
     // It's possible to force this by setting the env. variable
     // KDE_FORK_SLAVES, Clearcase seems to require this.
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     if (bForkSlaves.load() == -1) {
+#else
+    if (bForkSlaves.loadRelaxed() == -1) {
+#endif
         bool fork = qEnvironmentVariableIsSet("KDE_FORK_SLAVES");
 
         // no dbus? => fork slaves as we can't talk to klauncher
@@ -113,7 +117,11 @@ static bool forkSlaves()
 
         bForkSlaves.testAndSetRelaxed(-1, fork ? 1 : 0);
     }
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     return bForkSlaves.load() == 1;
+#else
+    return bForkSlaves.loadRelaxed() == 1;
+#endif
 }
 
 namespace KIO
@@ -513,7 +521,13 @@ Slave *Slave::createSlave(const QString &protocol, const QUrl &url, int &error,
             << qlibexec
             << qlibexecKF5
             << QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5); // look at our installation location
-        const QString kioslaveExecutable = QStandardPaths::findExecutable(QStringLiteral("kioslave5"), searchPaths);
+        QString kioslaveExecutable = QStandardPaths::findExecutable(QStringLiteral("kioslave5"), searchPaths);
+        if (kioslaveExecutable.isEmpty()) {
+            // Fallback to PATH. On win32 we install to bin/ which tests outside
+            // KIO cannot not find at the time ctest is run because it
+            // isn't the same as applicationDirPath().
+            kioslaveExecutable = QStandardPaths::findExecutable(QStringLiteral("kioslave5"));
+        }
         if (kioslaveExecutable.isEmpty()) {
             error_text = i18n("Can not find 'kioslave5' executable at '%1'", searchPaths.join(QLatin1String(", ")));
             error = KIO::ERR_CANNOT_CREATE_SLAVE;
