@@ -41,12 +41,11 @@
 #include <qsavefile.h>
 #include <QDebug>
 
-#include <QtCore/QString>
-#include <QtCore/QFile>
-#include <QtCore/QDir>
-#include <QtCore/QRegExp>
-#include <QtCore/QTextStream>
-#include <QtCore/QLocale>
+#include <QString>
+#include <QFile>
+#include <QRegularExpression>
+#include <QTextStream>
+#include <QLocale>
 #include <QUrl>
 
 Q_LOGGING_CATEGORY(KIO_COOKIEJAR, "kf5.kio.cookiejar")
@@ -74,7 +73,7 @@ static QString removeWeekday(const QString &value)
     const int index = value.indexOf(QL1C(' '));
     if (index > -1) {
         int pos = 0;
-        const QString weekday = value.left(index);
+        const QStringRef weekday = value.leftRef(index);
         const QLocale cLocale = QLocale::c();
         for (int i = 1; i < 8; ++i) {
             // No need to check for long names since the short names are
@@ -119,6 +118,7 @@ static QDateTime parseDate(const QString &_value)
         for (int i = 0; date_formats[i]; ++i) {
             dt = cLocale.toDateTime(value, QL1S(date_formats[i]));
             if (dt.isValid()) {
+                dt.setTimeSpec(Qt::UTC);
                 break;
             }
         }
@@ -238,7 +238,7 @@ QString KHttpCookie::cookieStr(bool useDOMFormat) const
                 result += QL1S("; $Port");
             } else {
                 QString portNums;
-                Q_FOREACH (int port, mPorts) {
+                for (int port : qAsConst(mPorts)) {
                     portNums += QString::number(port) + QL1C(' ');
                 }
                 result += QL1S("; $Port=\"") + portNums.trimmed() + QL1C('"');
@@ -259,7 +259,7 @@ bool KHttpCookie::match(const QString &fqdn, const QStringList &domains,
             return false;
         }
     } else if (!domains.contains(mDomain)) {
-        if (mDomain[0] == '.') {
+        if (mDomain[0] == QL1C('.')) {
             return false;
         }
 
@@ -345,7 +345,7 @@ static void removeDuplicateFromList(KHttpCookieList *list, KHttpCookie &cookiePt
         if (cookiePtr.name() == cookie.name() &&
                 (nameMatchOnly || (domain1 == domain2 && cookiePtr.path() == cookie.path()))) {
             if (updateWindowId) {
-                Q_FOREACH (WId windowId, cookie.windowIds()) {
+                for (WId windowId : cookie.windowIds()) {
                     if (windowId && (!cookiePtr.windowIds().contains(windowId))) {
                         cookiePtr.windowIds().append(windowId);
                     }
@@ -447,7 +447,7 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, WId wind
     }
 
     int protVersion = 0;
-    Q_FOREACH (const KHttpCookie &cookie, allCookies) {
+    for (const KHttpCookie &cookie : qAsConst(allCookies)) {
         if (cookie.protocolVersion() > protVersion) {
             protVersion = cookie.protocolVersion();
         }
@@ -459,14 +459,14 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, WId wind
         }
 
         if (protVersion > 0) {
-            cookieStr = cookieStr + QStringLiteral("$Version=") + QString::number(protVersion) + QStringLiteral("; ");
+            cookieStr = cookieStr + QLatin1String("$Version=") + QString::number(protVersion) + QLatin1String("; ");
         }
 
-        Q_FOREACH (const KHttpCookie &cookie, allCookies) {
+        for (const KHttpCookie &cookie : qAsConst(allCookies)) {
             cookieStr = cookieStr + cookie.cookieStr(useDOMFormat) + QStringLiteral("; ");
         }
 
-        cookieStr.truncate(cookieStr.length() - 2); // Remove the trailing ';'
+        cookieStr.chop(2); // Remove the trailing '; '
     }
 
     return cookieStr;
@@ -568,7 +568,7 @@ void KCookieJar::stripDomain(const QString &_fqdn, QString &_domain) const
     extractDomains(_fqdn, domains);
     if (domains.count() > 3) {
         _domain = domains[3];
-    } else if (domains.count() > 0) {
+    } else if (!domains.isEmpty()) {
         _domain = domains[0];
     } else {
         _domain = QL1S("");
@@ -624,12 +624,13 @@ void KCookieJar::extractDomains(const QString &_fqdn,
     }
 
     // Return numeric IPv6 addresses as is...
-    if (_fqdn[0] == '[') {
+    if (_fqdn[0] == QL1C('[')) {
         _domains.append(_fqdn);
         return;
     }
     // Return numeric IPv4 addresses as is...
-    if (_fqdn[0] >= '0' && _fqdn[0] <= '9' && _fqdn.indexOf(QRegExp(IP_ADDRESS_EXPRESSION)) > -1) {
+    if (_fqdn[0] >= QL1C('0') && _fqdn[0] <= QL1C('9')
+        && _fqdn.indexOf(QRegularExpression(QStringLiteral(IP_ADDRESS_EXPRESSION))) > -1) {
         _domains.append(_fqdn);
         return;
     }
@@ -641,7 +642,7 @@ void KCookieJar::extractDomains(const QString &_fqdn,
 
     QStringList partList = _fqdn.split(QL1C('.'), QString::SkipEmptyParts);
 
-    if (partList.count()) {
+    if (!partList.isEmpty()) {
         partList.erase(partList.begin());    // Remove hostname
     }
 
@@ -777,12 +778,12 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                 QString dom = Value.toLower();
                 // RFC2965 3.2.2: If an explicitly specified value does not
                 // start with a dot, the user agent supplies a leading dot
-                if (dom.length() && dom[0] != '.') {
-                    dom.prepend(".");
+                if (dom.length() > 0 && dom[0] != QL1C('.')) {
+                    dom.prepend(QL1C('.'));
                 }
                 // remove a trailing dot
-                if (dom.length() > 2 && dom[dom.length() - 1] == '.') {
-                    dom = dom.left(dom.length() - 1);
+                if (dom.length() > 2 && dom[dom.length() - 1] == QL1C('.')) {
+                    dom.chop(1);
                 }
 
                 if (dom.count(QL1C('.')) > 1 || dom == QLatin1String(".local")) {
@@ -836,7 +837,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                 } else {
                     bool ok;
                     const QStringList portNums = Value.split(QL1C(' '), QString::SkipEmptyParts);
-                    Q_FOREACH (const QString &portNum, portNums) {
+                    for (const QString &portNum : portNums) {
                         const int port = portNum.toInt(&ok);
                         if (ok) {
                             lastCookie.mPorts.append(port);
@@ -1010,14 +1011,14 @@ void KCookieJar::addCookie(KHttpCookie &cookie)
         cookieList->push_back(cookie);
         // Use a stable sort so that unit tests are reliable.
         // In practice it doesn't matter though.
-        qStableSort(cookieList->begin(), cookieList->end(), compareCookies);
+        std::stable_sort(cookieList->begin(), cookieList->end(), compareCookies);
 
         m_cookiesChanged = true;
     }
 }
 
 //
-// This function advices whether a single KHttpCookie object should
+// This function advises whether a single KHttpCookie object should
 // be added to the cookie jar.
 //
 KCookieAdvice KCookieJar::cookieAdvice(const KHttpCookie &cookie) const
@@ -1098,9 +1099,8 @@ KCookieAdvice KCookieJar::getDomainAdvice(const QString &_domain) const
 // This function sets the advice for all cookies originating from
 // _domain.
 //
-void KCookieJar::setDomainAdvice(const QString &_domain, KCookieAdvice _advice)
+void KCookieJar::setDomainAdvice(const QString &domain, KCookieAdvice _advice)
 {
-    QString domain(_domain);
     KHttpCookieList *cookieList = m_cookieDomains.value(domain);
 
     if (cookieList) {
@@ -1181,7 +1181,7 @@ KHttpCookieList *KCookieJar::getCookieList(const QString &_domain,
 // Eat a cookie out of the jar.
 // cookieIterator should be one of the cookies returned by getCookieList()
 //
-void KCookieJar::eatCookie(KHttpCookieList::iterator cookieIterator)
+void KCookieJar::eatCookie(const KHttpCookieList::iterator &cookieIterator)
 {
     const KHttpCookie &cookie = *cookieIterator;
     const QString domain = stripDomain(cookie); // We file the cookie under this domain.
@@ -1222,15 +1222,17 @@ void KCookieJar::eatSessionCookies(long windowId)
         return;
     }
 
-    Q_FOREACH (const QString &domain, m_domainList) {
+    for (const QString &domain : qAsConst(m_domainList)) {
         eatSessionCookies(domain, windowId, false);
     }
 }
 
 void KCookieJar::eatAllCookies()
 {
-    Q_FOREACH (const QString &domain, m_domainList) {
-        eatCookiesForDomain(domain);    // This might remove domain from m_domainList!
+    // we need a copy as eatCookiesForDomain() might remove domain from m_domainList
+    const QStringList list = m_domainList;
+    for (const QString &domain : list) {
+        eatCookiesForDomain(domain);
     }
 }
 
@@ -1283,7 +1285,7 @@ static QString hostWithPort(const KHttpCookie *cookie)
     }
 
     QStringList portList;
-    Q_FOREACH (int port, ports) {
+    for (int port : ports) {
         portList << QString::number(port);
     }
 
@@ -1292,7 +1294,7 @@ static QString hostWithPort(const KHttpCookie *cookie)
 
 //
 // Saves all cookies to the file '_filename'.
-// On succes 'true' is returned.
+// On success 'true' is returned.
 // On failure 'false' is returned.
 bool KCookieJar::saveCookies(const QString &_filename)
 {
@@ -1306,11 +1308,11 @@ bool KCookieJar::saveCookies(const QString &_filename)
 
     ts << "# KDE Cookie File v2\n#\n";
 
-    QString s;
-    s.sprintf("%-20s %-20s %-12s %-10s %-4s %-20s %-4s %s\n",
+    const QString str =
+        QString::asprintf("%-20s %-20s %-12s %-10s %-4s %-20s %-4s %s\n",
               "# Host", "Domain", "Path", "Exp.date", "Prot",
               "Name", "Sec", "Value");
-    ts << s.toLatin1().constData();
+    ts << str;
 
     QStringListIterator it(m_domainList);
     while (it.hasNext()) {
@@ -1338,12 +1340,12 @@ bool KCookieJar::saveCookies(const QString &_filename)
                     ts << '[' << domain.toLocal8Bit().data() << "]\n";
                 }
                 // Store persistent cookies
-                const QString path = QL1S("\"") + cookie.path() + QL1C('"');
-                const QString domain = QL1S("\"") + cookie.domain() + QL1C('"');
+                const QString path = QL1C('"') + cookie.path() + QL1C('"');
+                const QString domain = QL1C('"') + cookie.domain() + QL1C('"');
                 const QString host = hostWithPort(&cookie);
 
                 // TODO: replace with direct QTextStream output ?
-                s.sprintf("%-20s %-20s %-12s %10lld  %3d %-20s %-4i %s\n",
+                const QString str = QString::asprintf("%-20s %-20s %-12s %10lld  %3d %-20s %-4i %s\n",
                           host.toLatin1().constData(), domain.toLatin1().constData(),
                           path.toLatin1().constData(), cookie.expireDate(),
                           cookie.protocolVersion(),
@@ -1351,7 +1353,7 @@ bool KCookieJar::saveCookies(const QString &_filename)
                           (cookie.isSecure() ? 1 : 0) + (cookie.isHttpOnly() ? 2 : 0) +
                           (cookie.hasExplicitPath() ? 4 : 0) + (cookie.name().isEmpty() ? 8 : 0),
                           cookie.value().toLatin1().constData());
-                ts << s.toLatin1().constData();
+                ts << str.toLatin1();
             }
         }
     }
@@ -1408,8 +1410,8 @@ static QString extractHostAndPorts(const QString &str, QList<int> *ports = nullp
     const QString host = str.left(index);
     if (ports) {
         bool ok;
-        QStringList portList = str.mid(index + 1).split(QL1C(','));
-        Q_FOREACH (const QString &portStr, portList) {
+        const QStringList list = str.mid(index + 1).split(QL1C(','));
+        for (const QString &portStr : list) {
             const int portNum = portStr.toInt(&ok);
             if (ok) {
                 ports->append(portNum);
@@ -1422,7 +1424,7 @@ static QString extractHostAndPorts(const QString &str, QList<int> *ports = nullp
 
 //
 // Reloads all cookies from the file '_filename'.
-// On succes 'true' is returned.
+// On success 'true' is returned.
 // On failure 'false' is returned.
 bool KCookieJar::loadCookies(const QString &_filename)
 {
@@ -1510,9 +1512,9 @@ bool KCookieJar::loadCookies(const QString &_filename)
                 continue;
             }
 
-            KHttpCookie cookie(host, domain, path, name, value, expDate,
+            KHttpCookie cookie(host, domain, path, name, QString::fromUtf8(value), expDate,
                                protVer, secure, httpOnly, explicitPath);
-            if (ports.count()) {
+            if (!ports.isEmpty()) {
                 cookie.mPorts = ports;
             }
             addCookie(cookie);
@@ -1577,7 +1579,9 @@ void KCookieJar::loadConfig(KConfig *_config, bool reparse)
     m_globalAdvice = strToAdvice(policyGroup.readEntry("CookieGlobalAdvice", QStringLiteral("Accept")));
 
     // Reset current domain settings first.
-    Q_FOREACH (const QString &domain, m_domainList) {
+    // We need a copy as setDomainAdvice() may modify m_domainList
+    const QStringList list = m_domainList;
+    for (const QString &domain : list) {
         setDomainAdvice(domain, KCookieDunno);
     }
 
@@ -1604,7 +1608,7 @@ QDebug operator<<(QDebug dbg, const KHttpCookie &cookie)
 
 QDebug operator<<(QDebug dbg, const KHttpCookieList &list)
 {
-    Q_FOREACH (const KHttpCookie &cookie, list) {
+    for (const KHttpCookie &cookie : list) {
         dbg << cookie;
     }
     return dbg;
