@@ -28,7 +28,6 @@
 // KDE
 #include <QLineEdit>
 #include <kcombobox.h>
-#include <KConfigGroup>
 #include <QVBoxLayout>
 
 class DomainNameValidator : public QValidator
@@ -43,14 +42,14 @@ public:
 
     State validate (QString& input, int&) const override
     {
-        if (input.isEmpty() || (input == QLatin1String("."))) {
+        if (input.isEmpty() || (input == QLatin1Char('.'))) {
             return Intermediate;
         }
 
         const int length = input.length();
 
         for (int i = 0 ; i < length; i++) {
-            if (!input[i].isLetterOrNumber() && input[i] != '.' && input[i] != '-') {
+            if (!input[i].isLetterOrNumber() && input[i] != QLatin1Char('.') && input[i] != QLatin1Char('-')) {
                 return Invalid;
             }
         }
@@ -79,10 +78,10 @@ KCookiesPolicySelectionDlg::KCookiesPolicySelectionDlg (QWidget* parent, Qt::Win
     connect(mButtonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
 
     mButtonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
-    connect(mUi.leDomain, SIGNAL(textEdited(QString)),
-            SLOT(slotTextChanged(QString)));
-    connect(mUi.cbPolicy, SIGNAL(currentIndexChanged(QString)),
-            SLOT(slotPolicyChanged(QString)));
+    connect(mUi.leDomain, &QLineEdit::textEdited,
+            this, &KCookiesPolicySelectionDlg::slotTextChanged);
+    connect(mUi.cbPolicy, QOverload<const QString&>::of(&QComboBox::currentIndexChanged),
+            this, &KCookiesPolicySelectionDlg::slotPolicyChanged);
 
     mUi.leDomain->setFocus();
 }
