@@ -38,6 +38,7 @@
 #include <QStyleOptionButton>
 #include <qstandardpaths.h>
 #include <qmimedatabase.h>
+#include <QScreen>
 
 #include <kurlauthorized.h>
 #include <khistorycombobox.h>
@@ -81,6 +82,8 @@ public:
     {
         qDeleteAll(children);
     }
+    AppNode(const AppNode &) = delete;
+    AppNode &operator=(const AppNode &) = delete;
 
     QString icon;
     QString text;
@@ -141,14 +144,12 @@ void KApplicationModelPrivate::fillNode(const QString &_entryPath, KDEPrivate::A
 
     const KServiceGroup::List list = root->entries();
 
-    for (KServiceGroup::List::ConstIterator it = list.begin();
-            it != list.end(); ++it) {
+    for (const KSycocaEntry::Ptr &p : list) {
         QString icon;
         QString text;
         QString entryPath;
         QString exec;
         bool isDir = false;
-        const KSycocaEntry::Ptr p = (*it);
         if (p->isType(KST_KService)) {
             const KService::Ptr service(static_cast<KService*>(p.data()));
 
@@ -185,7 +186,7 @@ void KApplicationModelPrivate::fillNode(const QString &_entryPath, KDEPrivate::A
         newnode->parent = node;
         node->children.append(newnode);
     }
-    qStableSort(node->children.begin(), node->children.end(), KDEPrivate::AppNodeLessThan);
+    std::stable_sort(node->children.begin(), node->children.end(), KDEPrivate::AppNodeLessThan);
 }
 
 KApplicationModel::KApplicationModel(QObject *parent)
@@ -268,7 +269,7 @@ void KApplicationModel::fetchAll(const QModelIndex &parent)
 
     int childCount = rowCount(parent);
     for (int i = 0; i < childCount; i++) {
-        const QModelIndex &child = parent.child(i, 0);
+        const QModelIndex &child = index(i, 0, parent);
         // Recursively call the function for each child node.
         fetchAll(child);
     }
@@ -395,14 +396,6 @@ bool QTreeViewProxyFilter::filterAcceptsRow(int sourceRow, const QModelIndex &pa
         return true;
     }
 
-    //Show the non-leaf node also if the regexp matches one of its children
-    int rows = sourceModel()->rowCount(index);
-    for (int crow = 0; crow < rows; crow++) {
-        if (filterAcceptsRow(crow, index)) {
-            return true;
-        }
-    }
-
     return false;
 }
 
@@ -545,6 +538,7 @@ public:
     QLabel *label;
     QString qMimeType;
     QString qMimeTypeComment;
+    KCollapsibleGroupBox *dialogExtension;
     QCheckBox *terminal;
     QCheckBox *remember;
     QCheckBox *nocloseonexit;
@@ -710,6 +704,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     proxyModel->setSourceModel(appModel);
     proxyModel->setFilterKeyColumn(0);
     proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
+    proxyModel->setRecursiveFilteringEnabled(true);
     view = new KApplicationView(q);
     view->setModels(appModel, proxyModel);
     topLayout->addWidget(view);
@@ -731,11 +726,11 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     }
 
     //Advanced options
-    KCollapsibleGroupBox *dialogExtension = new KCollapsibleGroupBox(q);
+    dialogExtension = new KCollapsibleGroupBox(q);
     dialogExtension->setTitle(i18n("Terminal options"));
 
     QVBoxLayout *dialogExtensionLayout = new QVBoxLayout;
-    dialogExtensionLayout->setMargin(0);
+    dialogExtensionLayout->setContentsMargins(0, 0, 0, 0);
 
     terminal = new QCheckBox(i18n("Run in &terminal"), q);
     if (bReadOnly) {
@@ -751,7 +746,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     checkBoxIndentation += terminal->style()->pixelMetric(QStyle::PM_CheckBoxLabelSpacing, &checkBoxOption, terminal);
 
     QBoxLayout *nocloseonexitLayout = new QHBoxLayout();
-    nocloseonexitLayout->setMargin(0);
+    nocloseonexitLayout->setContentsMargins(0, 0, 0, 0);
     QSpacerItem *spacer = new QSpacerItem(checkBoxIndentation, 0, QSizePolicy::Fixed, QSizePolicy::Minimum);
     nocloseonexitLayout->addItem(spacer);
 
@@ -764,7 +759,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     KConfigGroup confGroup(KSharedConfig::openConfig(), QStringLiteral("General"));
     QString preferredTerminal = confGroup.readPathEntry("TerminalApplication", QStringLiteral("konsole"));
 
-    if (bReadOnly || preferredTerminal != QStringLiteral("konsole")) {
+    if (bReadOnly || preferredTerminal != QLatin1String("konsole")) {
         nocloseonexit->hide();
     }
 
@@ -784,7 +779,11 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     //edit->setText( _value );
     // The resize is what caused "can't click on items before clicking on Name header" in previous versions.
     // Probably due to the resizeEvent handler using width().
-    q->resize( q->minimumWidth(), 0.6*QApplication::desktop()->availableGeometry().height());
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    q->resize( q->minimumWidth(), 0.6 * QApplication::screens().at(0)->availableGeometry().height());
+#else
+    q->resize( q->minimumWidth(), 0.6 * q->screen()->availableGeometry().height());
+#endif
     edit->setFocus();
     q->slotTextChanged();
 }
@@ -836,10 +835,11 @@ void KOpenWithDialog::slotTextChanged()
     //If the search string doesn't match anything there will be no nodes to expand
     if (d->edit->text().size() > 2) {
         d->view->expandAll();
+        QAbstractItemModel *model = d->view->model();
         //Automatically select the first result (first leaf node) when the filter has match
-        QModelIndex leafNodeIdx = d->view->model()->index(0, 0);
-        while (d->view->model()->hasChildren(leafNodeIdx)) {
-            leafNodeIdx = leafNodeIdx.child(0,0);
+        QModelIndex leafNodeIdx = model->index(0, 0);
+        while (model->hasChildren(leafNodeIdx)) {
+            leafNodeIdx = model->index(0, 0, leafNodeIdx);
         }
         d->view->setCurrentIndex(leafNodeIdx);
     } else {
@@ -999,7 +999,7 @@ bool KOpenWithDialogPrivate::checkAccept()
         preferredTerminal = confGroup.readPathEntry("TerminalApplication", QStringLiteral("konsole"));
         m_command = preferredTerminal;
         // only add --noclose when we are sure it is konsole we're using
-        if (preferredTerminal == QStringLiteral("konsole") && nocloseonexit->isChecked()) {
+        if (preferredTerminal == QLatin1String("konsole") && nocloseonexit->isChecked()) {
             m_command += QStringLiteral(" --noclose");
         }
         m_command += QLatin1String(" -e ") + edit->text();
@@ -1076,9 +1076,8 @@ bool KOpenWithDialogPrivate::checkAccept()
 
             if (!qMimeType.isEmpty()) {
                 addToMimeAppsList(menuId);
-            } else {
-                m_pService = new KService(newPath);
             }
+            m_pService = new KService(newPath);
         }
     }
 
@@ -1130,6 +1129,8 @@ void KOpenWithDialog::hideNoCloseOnExit()
     // uncheck the checkbox because the value could be used when "Run in Terminal" is selected
     d->nocloseonexit->setChecked(false);
     d->nocloseonexit->hide();
+
+    d->dialogExtension->setVisible(d->nocloseonexit->isVisible() || d->terminal->isVisible());
 }
 
 void KOpenWithDialog::hideRunInTerminal()
