@@ -29,7 +29,6 @@
 #include <KPluginMetaData>
 
 #include <QHashIterator>
-#include <QStringBuilder>
 #include <QHostInfo>
 #include <QHostAddress>
 
@@ -46,7 +45,7 @@ static QString lookupIconNameFor(const QUrl &url, KUriFilterData::UriTypes type)
         break;
     case KUriFilterData::Executable: {
         QString exeName = url.path();
-        exeName = exeName.mid(exeName.lastIndexOf('/') + 1);     // strip path if given
+        exeName.remove(0, exeName.lastIndexOf(QLatin1Char('/')) + 1);     // strip path if given
         KService::Ptr service = KService::serviceByDesktopName(exeName);
         if (service && service->icon() != QLatin1String("unknown")) {
             iconName = service->icon();
@@ -179,7 +178,7 @@ public:
           wasModified(true),
           uriType(KUriFilterData::Unknown),
           searchFilterOptions(KUriFilterData::SearchFilterOptionNone),
-          url(u),
+          url(u.adjusted(QUrl::NormalizePathSegments)),
           typedString(typedUrl)
     {
     }
@@ -195,7 +194,7 @@ public:
         uriType = KUriFilterData::Unknown;
         searchFilterOptions = KUriFilterData::SearchFilterOptionNone;
 
-        url = u;
+        url = u.adjusted(QUrl::NormalizePathSegments);
         typedString = typedUrl;
 
         errMsg.clear();
@@ -497,7 +496,7 @@ QString KUriFilterPlugin::configName() const
 
 void KUriFilterPlugin::setFilteredUri(KUriFilterData &data, const QUrl &uri) const
 {
-    data.d->url = uri;
+    data.d->url = uri.adjusted(QUrl::NormalizePathSegments);
     data.d->wasModified = true;
     //qDebug() << "Got filtered to:" << uri;
 }
@@ -531,7 +530,8 @@ void KUriFilterPlugin::setSearchProvider(KUriFilterData &data, const QString &pr
 
 void KUriFilterPlugin::setSearchProviders(KUriFilterData &data, const QList<KUriFilterSearchProvider *> &providers) const
 {
-    Q_FOREACH (KUriFilterSearchProvider *searchProvider, providers) {
+    data.d->searchProviderList.reserve(data.d->searchProviderList.size() + providers.size());
+    for (KUriFilterSearchProvider *searchProvider : providers) {
         data.d->searchProviderList << searchProvider->name();
         data.d->searchProviderMap.insert(searchProvider->name(), searchProvider);
     }
@@ -589,7 +589,7 @@ bool KUriFilter::filterUri(KUriFilterData &data, const QStringList &filters)
 {
     bool filtered = false;
 
-    for (KUriFilterPlugin *plugin : d->pluginList) {
+    for (KUriFilterPlugin *plugin : qAsConst(d->pluginList)) {
         // If no specific filters were requested, iterate through all the plugins.
         // Otherwise, only use available filters.
         if (filters.isEmpty() || filters.contains(plugin->objectName())) {
@@ -636,12 +636,10 @@ QString KUriFilter::filteredUri(const QString &uri, const QStringList &filters)
     return data.uri().toString();
 }
 
-#ifndef KIOWIDGETS_NO_DEPRECATED
 bool KUriFilter::filterSearchUri(KUriFilterData &data)
 {
     return filterSearchUri(data, (NormalTextFilter | WebShortcutFilter));
 }
-#endif
 
 bool KUriFilter::filterSearchUri(KUriFilterData &data, SearchFilterTypes types)
 {
@@ -668,16 +666,17 @@ QStringList KUriFilter::pluginNames() const
 
 void KUriFilter::loadPlugins()
 {
-    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins("kf5/urifilters");
+    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/urifilters"));
+    const QString prefKey = QStringLiteral("X-KDE-InitialPreference");
     // Sort the plugins by order of priority
-    std::sort(plugins.begin(), plugins.end(), [](const KPluginMetaData &a, const KPluginMetaData &b) {
-            return a.rawData().value("X-KDE-InitialPreference").toInt() > b.rawData().value("X-KDE-InitialPreference").toInt();
+    std::sort(plugins.begin(), plugins.end(), [prefKey](const KPluginMetaData &a, const KPluginMetaData &b) {
+            return a.rawData().value(prefKey).toInt() > b.rawData().value(prefKey).toInt();
     });
 
     QStringList pluginNames;
     pluginNames.reserve(plugins.count());
 
-    for (const KPluginMetaData &pluginMetaData : plugins) {
+    for (const KPluginMetaData &pluginMetaData : qAsConst(plugins)) {
         const QString fileName = pluginMetaData.fileName().section(QLatin1Char('/'), -1);
         if (!pluginNames.contains(fileName)) {
             pluginNames << fileName;
